function [X,Y,Lab] = extractWindows(origdata,labels,windowlength_sec,overlapfactor,samplfreq,minfreq,maxfreq)

% Generation of epochs and computation of the short-time Fourier-transform
% of the input data.
%
% Inputs:
%
% origdata - input data is either matrix of size C*N, or tensor of size C*N*T,
% where C = number of channels, N = number of epochs, T = number of time points.
% labels - N-element vector denoting category of each sample.
% Each epoch can only belong to one category and thus epochs collapsing
% between two categories are not accepted.
% windowlength_sec - epoch length in seconds
% overlapfactor - overlap factor between adjacent epochs (1 = no overlap)
% samplfreq - sampling frequency in Hz
% minfreq - minimum frequency in Hz
% maxfreq - maximum frequency in Hz
%
% Outputs:
%
% X - three-way data containing Fourier transforms of epochs.
% Size of X is C*N*F, where F = number of frequency bins, N = number of epochs, C = number of channels
% Y - three-way data containing time-domain epochs.
% Size of Y is C*N*T, where T = number of time points, N = number of epochs, C = number of channels
% Lab - category labels of the epochs
%
% Last modified 12.11.2016
%
% See also:
% PREPROCESS_DATA
% FOURIERICA_CUSTOM


hammingdata = 0;

if length(size(origdata)) == 3 % data is in epoch format
    Y = origdata;
    origdata = permute(origdata,[3 2 1]);
    windowsize = size(origdata,1);
    windowlength_sec = windowsize/samplfreq;
    windows = size(origdata,2);
    channels = size(origdata,3);
    
    %compute fft indices
    startfftind = floor(minfreq*windowlength_sec+1);
    endfftind = floor(maxfreq*windowlength_sec+1);
    fftsize = endfftind - startfftind + 1;
    if hammingdata
        hammingwindow = hamming(windowsize,'periodic');
    end
    %initialize frequency-data matrix
    X = zeros(fftsize,windows,channels);
    
    for j = 1:windows
        datawindow = squeeze(origdata(:,j,:))';
        % Multiply by a Hamming window if necessary:
        if hammingdata
            datawindow = datawindow*diag(hammingwindow);
        end
        datawindow_ft = fft(datawindow');
        X(:,j,:) = datawindow_ft(startfftind:endfftind,:);
    end
    Lab = labels;
    X = permute(X,[3 2 1]);
else % data in a matrix format channels*time (raw format)
    
    % Define size parameters
    channels = size(origdata,1); %number of channels
    T = size(origdata,2); %total number of time points in original data
    
    if isempty(labels)
        labels = NaN*zeros(1,T);
        stimuliStartInds = [1 length(labels)];
    else
        stimuliStartInds = [1 find(diff(labels)) + 1 length(labels)];
    end
    
    %Compute number of time points in one window based on other parameters
    windowsize = floor(windowlength_sec*samplfreq);
    %Compute interval between consecutive windowsample
    windowinterval = ceil(windowsize/overlapfactor);
    %Compute approximate number of windows. In practice real number of windows will
    % be smaller because border windows are removed.
    windows = floor((T-windowsize)/windowinterval+1);
    
    nrOfBlocks = length(stimuliStartInds) - 1;
    
    Lab = zeros(1,windows);
    g = 1;
    j = 1;
    %initialize time-domain matrix
    Y = zeros(windowsize,windows,channels);
    
    %compute fft indices
    startfftind = floor(minfreq*windowlength_sec+1);
    endfftind = floor(maxfreq*windowlength_sec+1);
    fftsize = endfftind - startfftind + 1;
    if hammingdata
        hammingwindow = hamming(windowsize,'periodic');
    end
    %initialize frequency-data matrix
    X = zeros(fftsize,windows,channels);
    
    for n = 1:nrOfBlocks
        window(1) = stimuliStartInds(n);
        window(2) = window(1) + windowsize - 1;
        d = 1;
        while ( window(2) <= stimuliStartInds(n+1) )
            datawindow = origdata(:,window(1):window(2));
            Y(:,j,:) = datawindow';
            
            % Multiply by a Hamming window if necessary:
            if hammingdata
                datawindow = datawindow*diag(hammingwindow);
            end
            datawindow_ft = fft(datawindow');
            X(:,j,:) = datawindow_ft(startfftind:endfftind,:);
            
            Lab(j) = labels(window(1));
            
            j = j + 1; % increase epoch index
            d = d + 1; % increase within-block running index
            
            window = window + windowinterval; % go to next time-interval
        end
    end
    
    finalNumberOfWindows = j - 1;
    X = X(:,1:finalNumberOfWindows,:);
    Y = Y(:,1:finalNumberOfWindows,:);
    Lab = Lab(1:finalNumberOfWindows);
    X = permute(X,[3 2 1]); % zeros(fftsize,windows,channels);
    Y = permute(Y,[3 2 1]); % zeros(fftsize,windows,channels);   
end
