function varargout = fourierica_custom2(origdata,varargin)

% FOURIER-ICA
%
% Simple code for doing FOURIER-ICA, based on the publication A. Hyvärinen, P. Ramkumar, L. Parkkonen,
% and R. Hari. Independent component analysis of short-time Fourier transforms for spontaneous
% EEG/MEG analysis. NeuroImage, 49(1):257-271, 2010.
%
% NOTE! THIS IS A SLIGHTLY MODIFIED VERSION OF THE ORIGINAL CODE RUNNING
% BASED ON MATRIX INPUT.
%
% Last modified: 1.5.2017
%
% See also:
% PREPROCESS_DATA
% EXTRACT_WINDOWS

%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input arguments are processed here. The code checks for available parameters and consistency. Only
% parameters specified in parfixnames (for mandatory parameters) or

if nargout<3 || nargout>5, error('Wrong number of output arguments.'), end

% Number of input arguments
numargin=size(varargin,2);

% Definition of mandatory parameters
parfixnames={'pcadim'};

% Definition of optional parameters, their types and default values
paroptnames={'showprogress','seed','lambda','conveps','complexmixing','windowlength_sec','overlapfactor','components', ...
    'removeoutliers','maxiter','fcnoutliers','zerotolerance','domain'};
paropttypes={'logical','numeric','numeric','numeric','logical','numeric','numeric','numeric','logical','numeric', ...
    'function_handle','numeric','char'};
paroptdefval={'true','-1','1','1e-7','0','1','2','pcadim','1','max(40*pcadim,2000)','0','conveps','''frequency'''};

% Convert input arguments into struct type
numparfix=size(parfixnames,2);
numparopt=size(paroptnames,2);

if numargin>=numparfix
    if all(isnumeric([varargin{1:numparfix}]))
        for indpar=1:numparfix, eval(['param.' parfixnames{indpar} '=varargin{indpar};']), end
        argini=numparfix+2;
    else
        argini=2;
    end
    if rem(numargin-argini,2)==1, error('Optional parameters should always go by pairs'); end
    for indarg=argini:2:numargin, if ischar(varargin{indarg-1}), ...
                eval(['param.' varargin{indarg-1} '=varargin{indarg};']), end, end
elseif numargin==1
    param=varargin{1};
else
    error('Wrong number of input arguments.');
end

% Assign input arguments to function parameters
numfields=0;

%fprintf('Launching Fourier-ICA\n')

messages={'PCA dimension set to '};

for indpar=1:numparfix,
    if isfield(param,parfixnames{indpar}),
        if isnumeric(param.([parfixnames{indpar}]))
            eval([ parfixnames{indpar} '=param.(parfixnames{indpar});'])
            messages{indpar,2}=param.(parfixnames{indpar});
        else
            error([ parfixnames{indpar} ' should be numeric']);
        end
        numfields=numfields+1;
    else
        error(['Field ' parfixnames{indpar} ' is mandatory.']);
    end
end

%disp(messages);

messages=cell(0,2);
for indpar=1:numparopt,
    if isfield(param,paroptnames{indpar}),
        numfields=numfields+1;
        if isa(param.(paroptnames{indpar}),paropttypes{indpar})
            eval([ paroptnames{indpar} '=param.(paroptnames{indpar});'])
            messages{numfields-1,1}=['Further: ' paroptnames{indpar} ' set to '];
            messages{numfields-1,2}=param.(paroptnames{indpar});
        else
            error([ paroptnames{indpar} ' should be ' paropttypes{indpar}]);
        end
    else
        eval([ paroptnames{indpar} '=' paroptdefval{indpar} ';'])
    end
end

%disp(messages);

warning off

if length(fieldnames(param))>numfields, ...
        warning('Some specified parameters were not recognized!'), end

warning on

clear param numargin varargin numfields paroptnames paroptdefval paropttypes parfixnames


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do PCA and whitening wrt to channels (this is compulsory) as typical in BSS algorithms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%disp('Spatially whitening data with PCA dimension reduction');
%"vectorize" data w.r.t. to last dimension (channels)
Xmat_c = origdata;
%store sample size after this transformation
N = size(Xmat_c,1);

%Substract mean value from channels
Xmat_c_mv = mean(Xmat_c);
Xmat_c = Xmat_c-Xmat_c_mv(ones(N,1),:);
clear Xmat_c_mv

%DO PCA AND WHITEN DATA (in matrix Xmat_c)
covmat=cov(Xmat_c);
if ~complexmixing, covmat=real(covmat); else covmat=conj(covmat); end
[Ec, Dc] = eig(covmat);
[d,order] = sort(diag(Dc),'descend');
clear covmat Dc

% Checks for negative values
if any(d(1:pcadim)<0), warning('Negative eigenvalues! Reducing PCA and ICA dimension...'), end
% Check for values near zero (relative to the maximum eigenvalue)
zeroeigval=sum((d(1:pcadim)/d(1))<zerotolerance);
% Adjust dimensions if necessary
pcadim=pcadim-zeroeigval;
if pcadim<components, components=pcadim; end
if zeroeigval, fprintf('PCA dimension is %d and ICA dimension is %d\n',pcadim,components); end

% Construct whitening and dewhitening matrices
dsqrt = sqrt(d(1:pcadim));
dsqrtinv = 1./dsqrt;
Ec = Ec(:,order(1:pcadim));
whiteningmatrix = diag(dsqrtinv)*Ec';
dewhiteningmatrix = Ec*diag(dsqrt);
clear d order zeroeigval dsqrt dsqrtinv Ec

% THIS MATRIX IS THE MAIN INPUT TO THE ITERATIVE ALGORITHMS:
Zmat_c = whiteningmatrix*transpose(Xmat_c);
Zmat_c_tr = Zmat_c'; % Also used in the fixed-point iteration.
clear Xmat_c

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FAST-ICA ESTIMATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%fprintf('Launching complex-valued FastICA:                              ');

% Sets the random number generator
switch seed
    case -1, numrandn=@randn;
    otherwise, stream=RandStream('mrg32k3a','seed',19); numrandn=@(x,y)randn(stream,x,y);
end

% Initial point, make it imaginary and unitary
switch complexmixing,
    case true, W_old=complex(numrandn(components,pcadim),numrandn(components,pcadim));
    case false, W_old=numrandn(components,pcadim);
end
W_old=sqrtm(inv(W_old*W_old'))*W_old;

% Iteration starts here

tic
for iter=1:maxiter
    %Compute outputs, note lack of conjugate
    Y=W_old*Zmat_c;
    
    %Computing nonlinearities
    Y2=abs(Y.*conj(Y));
    gY2 = 1./(lambda + Y2);
    dmv=lambda*sum(gY2.^2,2);
    
    %Fixed-point iteration
    W_new=(Y.*gY2)*Zmat_c_tr-diag(dmv)*W_old;
    
    %In case we want to restrict W to be real-valued, do it here:
    switch complexmixing, case true, W=W_new; case false, W=real(W_new); end
    
    %Make unitary
    W=sqrtm(inv(W*W'))*W;
    
    %check if converged
    convcriterion=1-sum(abs(sum(W.*conj(W_old),2)))/components;
    if convcriterion<conveps, break, end
    
    %Show progress to user
    if showprogress,
        erasechar={'\b'};
        infostr=[num2str(iter,'%4d') '    ' num2str(convcriterion,'%1.4e')];
        fprintf([erasechar{ones(length(infostr),1)} infostr]);
    end
    %store old value
    W_old=W;
end
T=toc;
clear Y Y2 gY2 dmv W_old W_new Zmat_c_tr

%Compute mixing matrix (in whitened space)
A=W';
%Compute source signal estimates
S=W*Zmat_c;

%Tell if convergence problems
if convcriterion>conveps,
%    fprintf(2,'\nFailed to converge, results may be wrong!\n')
else
%    fprintf(' Converged.\n')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sort components and transforms to original space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%fprintf('Sorting components and reformatting results.\n')

%compute objective function for each component
objective=-mean(log(lambda+abs(S.*conj(S))),2);

%sort components using the objective
[objective,componentorder]=sort(objective,'descend');
W=W(componentorder,:);
A=A(:,componentorder);
S=S(componentorder,:);

%Compute mixing and demixing matrix in original channel space
%Spatial filters
W_orig=W*whiteningmatrix;
%Spatial patterns
A_orig=dewhiteningmatrix*A;
%Independent components, but note that these are STFT's of the sources
S_FT = S;
clear S
%Output convergence data

convdata.convcriterion = convcriterion;
convdata.converged = convcriterion<conveps;
convdata.numiter = iter;
convdata.pcadim = pcadim;
convdata.components = components;

switch nargout
    case 3, varargout={S_FT,A_orig,W_orig};
    case 4, varargout={S_FT,A_orig,W_orig,objective};
    case 5, varargout={S_FT,A_orig,W_orig,objective,convdata};
    case 6, varargout={S_FT,A_orig,W_orig,objective,convdata,windows};
end

%fprintf('Done.\n')


