function [epochs,labels] = prepareDatasetsMEG(subject_index,Params)

% This function loads MEG (EEG) data based on provided input file names,
% and then preprocesses data and extracts epochs for classification purpose
% based on given epoching parameters.
%
% Inputs:
% subject_index - index of the subject (1,2,...)
% Params - parameter struct, created using init_decoding_params -function.
%
% Outputs:
% epochs - time-channel matrices of epochs of size (Nr epochs x Nr time points x Nr channels)
% labels - category label {1,2,...,C} of each epoch, where C is the number of categories
%
% See also:
% DECODING_MAIN
% RUN_DECODING
% INIT_DECODING_PARAMS

Params = update_directory_names(Params);

chanIndex = [];
delays_subj = []; 

epoch_dir = Params.privateParams.epoch_dirs{subject_index};

sourceFilename = Params.generalParams.sourceFilenames{subject_index};

classNames = Params.generalParams.classNames;
epochingParams = Params.epochingParams;

triggerChannels = Params.epochingParams.triggerChannels;

if exist([epoch_dir 'epochs.mat']) == 2 && Params.generalParams.overwrite == 0
    disp('Epochs found, loading them ....')
    load([epoch_dir 'epochs.mat'],'epochs','labels')
    return
end

if ~isempty(chanIndex)
    % EEG data used, perform laplacian.
    use_laplacian = 1;
    chanIndex = [];
else
    use_laplacian = 0;
end

if ~isempty(epochingParams.combineData)
    disp('Note! Leave-one-subject-out analysis will be run because epochingParams.combineData -field is non-empty.')
end
if ~isempty(epochingParams.combineData)
    if sum(epochingParams.combineData == subject_index) > 0
        error('In leave-one-subject-out analysis, subject index cannot coincide with any of the chosen subjects!')
    end
end

Fs = epochingParams.Fs;
winlen = epochingParams.winlen;
locut = epochingParams.lowcut;
highcut = epochingParams.highcut;
orde = epochingParams.order;
nofiltering = epochingParams.skipfiltering;
use_grad = epochingParams.use_grad;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check if multiple subjects are provided. If yes, perform leave-one-subject-out
% analysis. Otherwise, perform basic subject-wise analysis.
NC = 1;
if iscell(sourceFilename)
    if length(sourceFilename) > 1
        % concatenate data:
        NC = length(sourceFilename);
    end
end

for mm = 1:NC % loop in case of leave-one-subject-out analysis (NC > 1).
    if NC > 1
        pa = sourceFilename{mm};
    else
        pa = sourceFilename;
    end
    % load data:
    if use_laplacian % in case of EEG data, apply surface laplacian
        badEEG = return_bad_eeg(subj_prefix,session); % return bad channels
        % perform laplacian and separate MEG and EEG data:
        [data,labels,st1] = perform_Laplacian(sourceFilename,badEEG);
        data = data.EEG;
        disp('Surface Laplacian applied for EEG data.')
    else
        st1 = fiff_setup_read_raw(pa);
        data = fiff_read_raw_segment(st1);
    end
    
    % Compare original sampling rate (Fs_current) with the desired sampling rate (Fs)
    % and perform downsampling if needed.
    Fs_current = st1.info.sfreq;
    if Fs < Fs_current
        drop_sampling_rate = 1;
    end
    if Fs > Fs_current
        error(['Desired sampling rate Fs cannot be higher than the original sampling rate ' num2str(Fs_current) ' Hz!!'])
    end
    if Fs == Fs_current
        drop_sampling_rate = 0;
    end
    
    % find triggers used to locate epochs:
    if isempty(triggerChannels)
        % Stimulus trigger channels should always be provided to avoid problems in the analysis.
        % However, if they are not provided, we may find them automatically.
        warning('Trigger channels of different stimuli not provided by a user, trying to locate them automatically...')
        N = [];
        for Id = 307:size(data,1)
            Un = nonzeros(unique(data(Id,:)));
            if ~isempty(Un) && length(Un) > 1
                if norm(log2(Un) - round(log2(Un))) == 0
                    N = Id;
                    break
                end
            end
        end
        if isempty(N)
            error('Trigger channel not found!')
        end
        trig_val = data(N,:);
        trig_val_un = nonzeros(unique(trig_val));
        % find triggers which are used to locate epochs:
        triggers = logical(zeros(length(trig_val_un),size(trig_val,2)));
        for mm = 1:length(trig_val_un)
            triggers(mm,:) = trig_val == trig_val_un(mm);
        end
        disp(['Number of triggers found for different classes: ' num2str(sum(triggers,2)')])
        disp(' ')
        nrClasses = size(triggers,1);
        nrClasses2 = length(classNames);
        if nrClasses ~= nrClasses2
            error(['Trigger channels for ' num2str(nrClasses) ' classes found, but a user provided ' num2str(nrClasses2) ' classes!'])
        else
            disp(['Trigger channels for ' num2str(nrClasses) ' different stimulus categories found!'])
        end
    else
        ch_nam = st1.info.ch_names;
        tr_chan_ind = zeros(1,length(triggerChannels));
        for ch = 1:length(ch_nam)
            for tr = 1:length(triggerChannels)
                if strcmp(ch_nam{ch},triggerChannels{tr})
                    tr_chan_ind(tr) = ch;
                end
            end
        end
        triggers = data(tr_chan_ind,:) ~= 0;
        nrClasses = size(triggers,1);
    end
    
    % based on user input, pick either magnetometer data, gradiometer data, or combined data:
    if isempty(chanIndex)
        if use_grad == 1 % Find gradiometer channels:
            grad = find(arrayfun(@(x) x.unit,st1.info.chs) == 201);
            data = data(grad,:);
            disp(['Epochs created for ' num2str(length(grad)) ' gradiometers...'])
        elseif use_grad == 0 % Find magnetometer channels:
            magn = find(arrayfun(@(x) x.unit,st1.info.chs) == 112);
            data = data(magn,:);
            disp(['Epochs created for ' num2str(length(magn)) ' magnetometers...'])
        elseif use_grad == 2
            grad = find(arrayfun(@(x) x.unit,st1.info.chs) == 201);
            magn = find(arrayfun(@(x) x.unit,st1.info.chs) == 112);            
            combined = [grad magn];
            data = data(combined,:);
            disp(['Epochs created for ' num2str(length(magn)) ... 
                ' magnetometers and ' num2str(length(grad)) ' gradiometers...'])
        else
            error('Parameter "use_grad" must be 0,1, or 2!')
        end
    else
        data = data(chanIndex,:);
        disp(['Epochs created for custom channels provided by a user...'])
    end
    
    if drop_sampling_rate % down-sample data:
        R = Fs/Fs_current;
        [p,q] = rat(R);
        try
            data = resample(data',p,q)';
        catch
            data = resample(data',p,q)';
        end
        triggers = downsample(triggers',1/R)';
    end
    
    f = find(isnan(data(1,:)));
    if ~isempty(f)
        f = max(f) + 1;
        data = data(:,f:end);
        triggers = triggers(:,f:end);
    end
       
    % extract epochs:
    [epochs,labels,data_f,baselines] = create_epochs(data,triggers,...
        winlen,nofiltering,Fs,locut,highcut,orde,delays_subj);
    warning off;mkdir([epoch_dir]);warning on    

    if NC > 1
        if mm > 1
            epochs2 = cat(1,epochs2,epochs);
            labels2 = [labels2 labels];
        else
            epochs2 = epochs;
            labels2 = labels;
        end
    else
        epochs2 = epochs;
        labels2 = labels;
    end
end

epochs = epochs2;
labels = labels2;

disp('Saving epochs.....')
save([epoch_dir 'epochs.mat'],'epochs','labels')
%save([epoch_dir 'epochs.mat'],'epochs','labels','mean_epochs','max_mean_epochs','chan_max','chan_max_name','baselines')
