function results = run_decoding(epochs,labels,subject_index,Params)
% This function extracts epochs from original MEG time-series data,
% 
%
% Inputs:
% epochs - 3-D tensor of epochs. Size is (Nr epochs x Nr time points x Nr channels)
% labels - category label {1,2,...,C} of each epoch, where C is the number of categories
% subject_index - index of the subject (1,2,...)
% Params - parameter struct, created using init_decoding_params -function.
%
% Outputs:
% results - A struct containing classification results with following fields:
% 'RE': test accuracy of each CV-fold for each class pair. For instance,
% results.RE{1}(3) contains accuracy of the 1st class pair for 3rd CV fold.
% 'betas': classifier coefficients for each class pair and CV fold. For
% instance, results.betas{1}(:,3) contains the coefficients of 1st class pair
% and 3rd CV fold for each variable.
% 'V' and 'W' contain coefficients of the feature extraction process (if available).
% Dimensions of these coefficients depend on the decoding method.
% 'Em' - data in case of tensor decoding methods
% 'A' - spatial patterns, in case of ICA is run as a preprocessing step
%
% See also:
% DECODING_MAIN
% RUN_DECODING
% INIT_DECODING_PARAMS


tic

Params = update_directory_names(Params);

epoch_dir = Params.privateParams.epoch_dirs{subject_index};
ResFileName = Params.privateParams.ResFileName;

overwrite = Params.generalParams.overwrite;

decodingParams = Params.decodingParams;
epochingParams = Params.epochingParams;
generalParams = Params.generalParams;

if Params.privateParams.rankModel == 0
    disp(['Model: ' decodingParams.classifier ' classifier based on ' decodingParams.classifiername ' features'])
else
    disp(['Model: ' SLR classifier based on ' decodingParams.classifiername ' features'])    
end

% Check if the results are already computed and overwrite-option is
% "false", and in this case don't recompute the results.
if exist([epoch_dir ResFileName]) == 2
    disp('Results already found!')    
    if overwrite
        disp('Overwriting existing results...');
    else
       results = load([epoch_dir ResFileName],'RE','betas','A','Em','Params','V','W');
       [pairs,nrClasses,nrClassPairs,classPairnames] = returnClasspairIndex(Params.generalParams.classNames);
        for m = 1:length(classPairnames)
           disp(['Cross validation accuracy for subject ' num2str(subject_index) ', class pair ' classPairnames{m} ': ' num2str(round(10000*mean(results.RE{m}))/100) ' %'])
        end   
       return 
    end
else
    disp('Results not found, start classifier training ....')
    disp([''])
end

% subsample data if subsampling set by user:
if decodingParams.samplingfactor > 1
    un = unique(labels);
    sf = decodingParams.samplingfactor;
    disp(['Subsampling data by a factor of ' num2str(sf)])
    labels_tmp = [];
    epochs_tmp = [];
    for c = 1:length(un)
        fi = find(labels == un(c));
        labels_c = labels(fi);
        epochs_c = squeeze(epochs(fi,:,:));
        epochs_c = squeeze(epochs_c(1:sf:end,:,:));
        labels_c = labels_c(1:sf:end);
        epochs_tmp = cat(1,epochs_tmp,epochs_c);
        labels_tmp = [labels_tmp labels_c];
    end
    labels = labels_tmp;
    epochs = epochs_tmp;
    clear epochs_tmp labels_tmp
end
disp(['Data size: ' num2str(size(epochs))])

% Transform epochs to spectral format:
if Params.privateParams.spectralModel
    [epochsN,A] = transformEpochsToSpectralFormat(epochs,...
        labels(1,:),Params.decodingParams.nrICs,Params.epochingParams.Fs,Params.decodingParams.ica_on,decodingParams);
end
tim = [];
% perform necessary operations for temporal epochs:
if ( Params.privateParams.temporalModel | Params.privateParams.channelwiseTimeresolvedModel | Params.privateParams.timeresolvedModel )
    [epochs,A] = runPCAorICA(epochs,decodingParams.ica_on,decodingParams.nrICs,decodingParams.classifiername);
    epochsN = scaleEpochs(epochs,decodingParams.skipscaling);
    tim = linspace(0,epochingParams.winlen,size(epochs,2));
    % if channel-wise time-window based approach, keep max temporal resolution:
    if ~Params.privateParams.channelwiseTimeresolvedModel
        % drop temporal resolution if subwinlenSamp > 1:
        [epochsN,tim] = splitToTimeWindows(epochsN,decodingParams.subwinlenSamp,tim);
    end
end


[DAT,classifierType] = extractFeatures(epochsN,decodingParams.classifiername,decodingParams);


if Params.privateParams.channelwiseTimeresolvedModel
    [resu,Em] = trainChannelwiseTimeresolvedModel(labels,...
        DAT,classifierType,generalParams.pairs,decodingParams,decodingParams.classifierIndex);
else
    [resu,Em] = trainClassifier(labels,DAT,classifierType,...
        generalParams.pairs,decodingParams.classifiername,Params,decodingParams.classifierIndex);
end
% collect results:
if Params.privateParams.channelwiseTimeresolvedModel
    [RE,betas] = collectResultsChannelwiseTimeresolvedModel(...
        resu,decodingParams.multiclass,decodingParams.classifier);
elseif ( strcmp(Params.decodingParams.classifiername,'temporalbilinear') | ...
        strcmp(Params.decodingParams.classifiername,'bilinear') )
    RE = resu;
    betas = [];
    V = [];
    W = [];
else
    [RE,betas,V,W] = collectResultsClassifier(resu,decodingParams.multiclass,classifierType,decodingParams.classifier);
end

save([epoch_dir ResFileName],'RE','betas','A','Em','Params','V','W')
results = load([epoch_dir ResFileName],'RE','betas','A','Em','Params','V','W');

disp(['Finished after ' num2str(toc) ' seconds!'])

 [pairs,nrClasses,nrClassPairs,classPairnames] = returnClasspairIndex(Params.generalParams.classNames);
 for m = 1:length(classPairnames)
    disp(['Cross validation accuracy for subject ' num2str(subject_index) ', class pair ' classPairnames{m} ': ' num2str(round(10000*mean(results.RE{m}))/100) ' %'])
 end   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTIONS:

function [epochs,A] = transformEpochsToSpectralFormat(epochs,labels,...
    nrIC,Fs,ica_on,decodingParams)

minfreq = decodingParams.minfreq;
maxfreq = decodingParams.maxfreq;

A = [];
origdata = permute(epochs,[3 1 2]);
[X,Y,Lab] = extract_windows(origdata,labels,[],...
    [],Fs,minfreq,maxfreq);
if ica_on ~= 0 % run Fourier-ICA
    if ica_on == -1 % take PCA components only
        [S_FT,A,W_orig,Z_FT] = fourierica_custom(X,'pcadim',nrIC,'samplfreq',Fs,...
            'minfreq',minfreq,'maxfreq',maxfreq);
        epochsN = permute(Z_FT,[2 3 1]);
    else % take ICA components
        [S_FT,A,W_orig] = fourierica_custom(X,'pcadim',nrIC,'samplfreq',Fs,...
            'minfreq',minfreq,'maxfreq',maxfreq);
        epochsN = permute(S_FT,[2 3 1]);
    end
else % no Fourier-ICA
    epochsN = permute(X,[2 3 1]);
end

function epochsN = scaleEpochs(epochs,noscaling)

if noscaling
    epochsN = epochs;
else
    % normalize epochs:
    epochsN = zeros(size(epochs));
    for a = 1:size(epochs,1)
        for b = 1:size(epochs,3)
            epochsN(a,:,b) = squeeze(epochs(a,:,b))/norm(squeeze(epochs(a,:,b)));
        end
    end
end

function [epochs,A] = runPCAorICA(epochs,ica_on,nrIC,style)

A = [];
if strcmp(style,'temporal') % run PCA, take only one component.
    epochsP = permute(epochs,[2 1 3]);
    concatEpochs = reshape(epochsP,size(epochsP,1)*size(epochsP,2),size(epochsP,3))';
    [COEFF,SCORE,latent] = pca(concatEpochs'*1e10);
    prcomp = SCORE(:,1)';
    % back transformation to epoch format:
    epochs2 = reshape(prcomp',size(epochsP,1),size(epochsP,2),size(prcomp,1));
    epochs = permute(epochs2,[2 1 3]);
    A = COEFF(:,1);
else
    if ica_on ~= 0
        if ica_on == -1
            disp('Running PCA....')
            epochsP = permute(epochs,[2 1 3]);
            concatEpochs = reshape(epochsP,size(epochsP,1)*size(epochsP,2),size(epochsP,3))';
            % FOR TESTING ONLY: back transformation to epochs*time points*channels
            % epochsP2 = reshape(concatEpochs',size(epochsP,1),size(epochsP,2),size(epochsP,3));
            % epochs2 = permute(epochsP2,[2 1 3]);
            % isequal(epochs,epochs2)
            [COEFF,SCORE,latent] = pca(concatEpochs'*1e10);
            prcomp = SCORE(:,1:nrIC)';
            % back transformation to epoch format:
            epochs2 = reshape(prcomp',size(epochsP,1),size(epochsP,2),size(prcomp,1));
            epochs = permute(epochs2,[2 1 3]);
            A = COEFF(:,1:nrIC);
        else
            epochsP = permute(epochs,[2 1 3]);
            concatEpochs = reshape(epochsP,size(epochsP,1)*size(epochsP,2),size(epochsP,3))';
            data2 = concatEpochs*1e10;
            disp('Running Fast-ICA....')
            [ica, A, W] = fastica(data2,'lastEig',nrIC,'g','tanh','verbose','off');
            disp('Fast-ICA completed!')
            epochs2 = reshape(ica',size(epochsP,1),size(epochsP,2),size(ica,1)); % epokit
            epochs = permute(epochs2,[2 1 3]);
        end
    end
end

function [DAT,classifierType] = extractFeatures(epochsN,style,decodingParams)

classifierType = 'normal';

switch style
    case 'timeresolved'
        clear DAT
        epochsN2 = permute(epochsN,[1 3 2]);
        for n = 1:size(epochsN2,3)
            DAT{n} = epochsN2(:,:,n);
        end
    case 'temporal'
        clear DAT
        DAT{1} = epochsN;
    case 'temporalsplit'
        clear DAT E wins
        winlen = decodingParams.winlen_channelwisetimeresolved_samples;
        winstep = decodingParams.winstep_channelwisetimeresolved_samples;
        WinInds = 1:winlen;
        epochLen = size(epochsN,2);
        mm = 1;
        while WinInds(end) <= epochLen
            for nn = 1:size(epochsN,3) % for each channel
                DAT{mm,nn} = epochsN(:,WinInds,nn);
            end
            winStart = WinInds(1) + winstep;
            winEnd = WinInds(end) + winstep;
            WinInds = winStart:winEnd;
            mm = mm + 1;
        end
    case 'spatial'
        clear DAT
        %DAT{1} = (1/size(epochsN,2))*squeeze(sum(epochsN.^2,2));
        %DAT{1} = normalize_vectors(DAT{1},'rows');
        DAT{1} = (1/size(epochsN,2))*squeeze(sum(epochsN.^2,2));
        %  DAT{1} = (1/size(epochsN,2))*squeeze(mean(epochsN,2));
    case 'spatiotemporal'
        DAT{1} = reshape(epochsN,[size(epochsN,1) ...
            size(epochsN,2)*size(epochsN,3)]);
    case 'spectrospatial'
        DAT{1} = abs(reshape(epochsN,[size(epochsN,1) ...
            size(epochsN,2)*size(epochsN,3)]));
    case 'spectraltemporalLDA'
        DAT{1} = epochsST;
    case 'tf'
        
    case 'logspectralLDA'
        DAT{1} = log10(abs(epochsN));
        classifierType = 'temporalLDA';
    case 'spectral'
        DAT{1} = mean(abs(epochsN),3);
        classifierType = 'normal';
    case 'logspectral'
        DAT{1} = log10(mean(abs(epochsN),3));
        classifierType = 'normal';
    otherwise
        DAT{1} = epochsN;
        classifierType = style;
end

function [resu,Em] = trainChannelwiseTimeresolvedModel(labels,DAT,classifierType,pairs,decodingParams,classifierIndex)

if nargin < 6
    classifierIndex = [];
end

modelRank = decodingParams.modelRank;
multiclass = decodingParams.multiclass;
shuf = decodingParams.shuf;
sh_id = decodingParams.sh_id;

Em = [];

if ~isempty(classifierIndex)
    disp(['Total number of channels/components: ' num2str(size(DAT,2))])
    disp(['Computing channel/component ' num2str(classifierIndex)])
    for m = 1:size(DAT,1)
        DAT2{m,1} = DAT{m,classifierIndex};
    end
    DAT = DAT2;
end

clear resu
for m = 1:size(DAT,2)
    if isempty(classifierIndex)
        disp(['Training classifier for sensor (or IC):' num2str(m) '/' num2str(size(DAT,2))])
    end
    for z = 1:size(DAT,1)
        disp(['    Time-window: ' num2str(z) '/' num2str(size(DAT,1))])
        if multiclass
            D = DAT{z,m};
            resu{m}{z}.model = LASSO_stability_CV(D,labels,[],shuf,sh_id,classifierType,[],modelRank);
        else
            for n = 1:size(pairs,1)
                %                disp(['class-pair ' num2str(n)])
                
                labels2 = labels(1,:);
                fi = [find(labels2== pairs(n,1)) find(labels2== pairs(n,2))];
                D = DAT{z,m}(fi,:,:);
                Lab = labels2(fi);
                un = unique(Lab);
                LabTmp = Lab;
                for p = 1:2
                    LabTmp(Lab == un(p)) = p;
                end
                Lab = LabTmp;
                if size(labels,1) == 2 % add subject blocks in LOSO-case
                    Lab2 = labels(2,:);
                    Lab2 = Lab2(fi);
                    Lab = [Lab;Lab2];
                end
                warning off
                resu{m,n}{z}.model = LASSO_stability_CV(D,Lab,[],shuf,sh_id,classifierType,[],modelRank);
                % remove some fields to save memory:
                for c = 1:length(resu{m,n}{z}.model)
                    resu{m,n}{z}.model(c).ModelsCV = [];
                    resu{m,n}{z}.model(c).AccCV_all = [];
                    resu{m,n}{z}.model(c).labels_estim = [];
                    resu{m,n}{z}.model(c).labels_test = [];
                end
                warning on
                fclose all;
            end
        end
    end
end

function [resu,Em] = trainClassifier(labels,DAT,classifierType,pairs,style,Params,timepointIndex)

if nargin < 7
    timepointIndex = [];
end

decodingParams = Params.decodingParams;
generalParams = Params.generalParams;

modelRank = decodingParams.modelRank;
multiclass = decodingParams.multiclass;

if generalParams.randomize
   shuf = 'on';
else
   shuf = 'off'; 
end

sh_id = generalParams.randomizeIndex;

classif = decodingParams.classifier;

Em = [];
if ~isempty(timepointIndex)  
    DAT2{1} = DAT{timepointIndex};
    DAT = DAT2;
end

clear resu
for m = 1:length(DAT)
    if strcmp(style,'timeresolved') && isempty(timepointIndex)
        disp(['Training classifier for time-interval ' num2str(m) ...
            '/' num2str(length(DAT))])
    end
    if multiclass
        D = DAT{m};
        Em = [];
        if ( strcmp(style,'threewayDA') | ...
                strcmp(style,'temporalthreewayDA')  | ...
                strcmp(style,'temporalthreewaySDA') | ...
                strcmp(style,'temporalthreewayRDA') | ...
                strcmp(style,'temporalthreewayDAfullorig') | ...
                strcmp(style,'temporalthreewayDAfull') )
            D = {permute(D,[2 3 1])};
            clear Em
            un = unique(labels);
            for mm = 1:length(un)
                Em{mm,1} = squeeze(mean(D{1}(:,:,labels==un(mm)),3));
                Em{mm,2} = squeeze(mean(D{1}(:,:,labels~=un(mm)),3));
            end
            resu{m}.model = LASSO_stability_CV(D,labels,[],shuf,sh_id,classifierType,[],modelRank,classif);
        else
            resu{m}.model = LASSO_stability_CV(D,labels,[],shuf,sh_id,classifierType,[],modelRank,classif);
        end
    else
        
        [pairs,nrClasses,nrClassPairs,classPairnames] = returnClasspairIndex(Params.generalParams.classNames);
        for n = 1:size(pairs,1)
            if ~strcmp(style,'timeresolved')
                disp(['class pair ' num2str(n) '/' num2str(size(pairs,1)) ': ' classPairnames{n}])
            end
            fi = [find(labels(1,:) == pairs(n,1)) find(labels(1,:) == pairs(n,2))];
            D = DAT{m}(fi,:,:);
            Lab = labels(:,fi);
            un = unique(Lab(1,:));
            LabTmp = Lab(1,:);
            for p = 1:2
                LabTmp(Lab(1,:) == un(p)) = p;
            end
            if size(Lab,1) == 2
                LabTmp = [LabTmp;Lab(2,:)];
            end
            Lab = LabTmp;
            if ( strcmp(style,'threewayDA') | strcmp(style,'temporalthreewayDA') ...
                    | strcmp(style,'temporalthreewaySDA') | strcmp(style,'temporalbilinear') | ...
                    strcmp(style,'bilinear') | strcmp(style,'temporalthreewayRDA') | ...
                    strcmp(style,'temporalthreewayDAfull') | strcmp(style,'temporalthreewayDAfullorig') )
                D = {permute(D,[2 3 1])};
                clear Em
                Em{1,1} = squeeze(mean(D{1}(:,:,Lab(1,:)==1),3));
                Em{1,2} = squeeze(mean(D{1}(:,:,Lab(1,:)~=2),3));
            end
            resu{m,n}.model = LASSO_stability_CV(D,Lab,[],shuf,sh_id,classifierType,[],modelRank,classif);
        end
    end
end

function [RE,betas] = collectResultsChannelwiseTimeresolvedModel(resu,multiclass,classifierStyle)

if multiclass
    RE = [];
    clear betas
    for m = 1:size(resu,1)
        for n = 1:length(resu{m}.model)
            for z = 1:length(resu{m})
                RE(m,n,z) = resu{m}{z}.model(n).acc_test;
                bi = resu{m}{z}.model(n).bestInd;
                betas(m,n,z,:) = resu{m}{z}.model(n).fit.beta(:,bi);
            end
        end
    end
else
    RE = [];
    clear betas
    for p = 1:size(resu,2)
        for m = 1:size(resu,1) % kanavat
            for z = 1:length(resu{m,p}) % aikapisteiden lukum��r�
                for n = 1:length(resu{m,p}{z}.model) % CV folds
                    RE{p}(m,n,z) = resu{m,p}{z}.model(n).acc_test;
                    bi = resu{m,p}{z}.model(n).bestInd;
                    % betas: kanavat x CV folds x aikapist
                    % x painokertoimien lukum��r�
                    betas{p}(m,n,z,:) = resu{m,p}{z}.model(n).fit.beta(:,bi);
                end
            end
        end
    end
end

function [RE,betas,V,W] = collectResultsClassifier(resu,multiclass,classifierType,classifierStyle)

V = [];
W = [];
betas = [];
RE = [];
if multiclass
    for m = 1:size(resu,1)
        for n = 1:length(resu{m}.model)
            RE(m,n) = resu{m}.model(n).acc_test;
            B = resu{m}.model(n).fit.beta;
            ind = resu{m}.model(n).bestInd;
            for p = 1:length(B)
                betas{p,m}(:,n) = B{p}(:,ind);
            end
        end
    end
else
    RE = [];
    for p = 1:size(resu,2) % for each class pair p
        for m = 1:size(resu,1)
            for n = 1:length(resu{m,p}.model) % for each test fold n
                RE{p}(m,n) = resu{m,p}.model(n).acc_test;
                ind = resu{m,p}.model(n).bestInd;
                if ( strcmp(classifierType,'temporalLDA') | strcmp(classifierType,'temporalRLDA') ...
                        | strcmp(classifierType,'temporalLDAfull') | ...
                        strcmp(classifierType,'spectralLDA') | ...
                        strcmp(classifierType,'temporalICA') | ...
                        strcmp(classifierType,'temporalPCA') | ...
                        strcmp(classifierType,'spatialLDA') | ...
                        strcmp(classifierType,'spatialRLDA') | ...
                        strcmp(classifierType,'spatialLDAfull') )
                    V{p}(:,:,n) = squeeze(resu{m,p}.model(n).VVfinal);
                end
                if ( strcmp(classifierType,'temporalthreewayDA') | ...
                        strcmp(classifierType,'temporalthreewaySDA') | ...
                        strcmp(classifierType,'threewayDA') | ...
                        strcmp(classifierType,'temporalthreewayDAfull') | ...
                        strcmp(classifierType,'temporalthreewayDAfullorig') | ...
                        strcmp(classifierType,'temporalthreewayRDA') )
                    
                    V{p}(:,:,n) = resu{m,p}.model(n).UUfinal{1};
                    W{p}(:,:,n) = resu{m,p}.model(n).VVfinal{1};
                end
                switch classifierStyle
                    case 'SLR'
                        B = resu{m,p}.model(n).fit.beta;
                        betas{p,m}(:,n) = B(:,ind);
                    case 'SVM'
                        betas{p,m}(:,n) = resu{m,p}.model(n).fit(ind).mdl.Beta;
                    case 'LDA'
                        if isempty(resu{m,p}.model(n).fit(ind).mdl.Coeffs)
                            betas{p,m}(:,n) = NaN;
                        else
                            betas{p,m}(:,n) = resu{m,p}.model(n).fit(ind).mdl.Coeffs(2,1).Linear;
                        end
                end
            end
        end
    end
end

function plotResults(plotIndivRes,figOn,RE,Luokat,pairs,style,tim)

if plotIndivRes
    for m=1:length(RE);MEE(m)=mean(RE{m});end
    figure
    [so,soi] = sort(MEE,'Descend');
    bar(MEE(soi));grid on;title(style);ylabel('accuracy')
    clear Leg
    for m = 1:size(pairs,1)
        Leg{m} = [Luokat{pairs(m,1)} ' vs. ' Luokat{pairs(m,2)}];
    end
    
    set(gca,'XTick',1:length(Leg),'XTickLabel',Leg(soi),'XTickLabelRotation',45,'FontSize',5)
    ylim([max(0,min(MEE)-0.05) min(1,max(MEE)+0.05)])
end

if figOn
    mark{1} = 'ro-';mark{2} = 'gs-';
    mark{3} = 'b*-';mark{4} = 'md-';
    mark{5} = 'c^-';mark{6} = 'kv-';
    mark{7} = 'bo-';mark{8} = 'ms-';
    mark{9} = 'c*-';mark{10} = 'kd-';
    clear me_tot
    for p = 1:length(RE)
        me_tot(p) = max(mean(RE{p},2));
    end
    
    figure
    clear Nam
    ite = 1;
    nrClasses = length(Luokat);
    nrClassPairs = size(pairs,1);
    if nrClassPairs <= 6
        rows = 2;
        cols = 3;
    elseif nrClassPairs <= 28
        rows = 7;
        cols = 4;
    end
    for p = 1:length(RE)
        me = mean(RE{p},2);
        sqrtN = sqrt(size(RE{p},2));
        se = std(RE{p},[],2)/sqrtN;
        sh = 0;
        if sh == 1
            me1 = mean(RE{p},2);
            sqrtN1 = sqrt(size(RE{p},2));
            se1 = std(RE{p},[],2)/sqrtN1;
        end
        subplot(rows,cols,ite)
        plot(tim(1:length(me)),me,'k-','LineWidth',2);hold on
        plot(tim(1:length(me)),me-se,'g-')
        plot(tim(1:length(me)),me+se,'g-')
        ylim([0.2 1])
        if sh == 1
            plot(tim(1:length(me1)),me1,'c-','LineWidth',2);hold on
        end
        grid on;ylabel('accuracy');xlabel('aika (s)')
        plot(tim(1:length(me)),0.7*ones(1,length(me)),'LineWidth',1,'Color','r')
        plot(tim(1:length(me)),0.65*ones(1,length(me)),'LineWidth',1,'Color','b')
        plot(tim(1:length(me)),0.6*ones(1,length(me)),'LineWidth',1,'Color','m')
        clear Na
        title([Luokat{pairs(p,1)} ' vs ' Luokat{pairs(p,2)}]);
        ite = ite + 1;
        set(gca,'FontSize',6)
    end
    Nam{ite} = [Luokat{pairs(p,1)} ' vs ' Luokat{pairs(p,2)}];
    if sh == 1
        Na{1} = 'Task (mean)';
        Na{2} = 'Task (SEM)';
        Na{3} = 'Task (SEM)';
        Na{4} = 'Task permuted (mean)';
        %Na{5}='Task permuted (SEM)';
        %Na{6}='Task permuted (SEM)';
        Na{5}=['signif, p < 0.001'];
        Na{6}=['signif, p < 0.01'];
        Na{7}=['signif, p < 0.05'];
    else
        Na{1}='Task (mean)';
        Na{2}='Task (SEM)';
        Na{3}='Task (SEM)';
        Na{4}=['signif, p < 0.001'];
        Na{5}=['signif, p < 0.01'];
        Na{6}=['signif, p < 0.05'];
    end
    legend(Na)
end
