function [Data,Classifier_final,resu,relevantCoefficients,A] = sparse_bilinear_example(filename)

if nargin == 0
    dataset = 1; % synthetic 
elseif nargin == 1
    dataset = 2; % EMG
else
   error('Wrong number of input arguments!') 
end

% provide full pathname to the filename:
%filename = 'C:\Tutkimus\omat_paperit\EMG\PLOS\EMG_preproc_hand_subj1.mat';

% Regularization sequence of spatial (channel/component) coefficients.
% Higher values lead to more easily interpretable models but performance may degrade.
lambdas_comp = [10 20 30];
lambdas_freq = [0 10 20]; % regularization of frequency coefficients
modelRank = 1; % number of coefficient vectors

% Set path of the L1General package:
% addpath(genpath('C:\tutkimus\omat_paperit\EMG\PLOS\'))
% Before using this script, download LGeneral package from here:
% http://www.cs.ubc.ca/~schmidtm/Software/L1General.zip

if exist('L1General2_PSSgb.m') ~= 2
    error('Cannot proceed because L1General-package not found from Matlab path. The package is available in: http://www.cs.ubc.ca/~schmidtm/Software/L1General.zip')
end

rng(100,'twister');
relevantCoefficients = [];A = [];        

switch dataset
    case 1
        disp('Generating synthetic data...')
        % generate some synthetic data:
        F = 50; % number of frequency bins in each epoch
        P = 48; % number of sensors
        N = 1000; % number of epochs
        K = 2; % number of categories
        
        labels = 1 + floor(K*rand(1,N));
        data = rand(F,P,N);
        for m = 1:K
            rnCol = randperm(size(data,2));
            data(:,rnCol(1:4),labels == m) = 0.1*m + data(:,rnCol(1:4),labels == m);
        end
        data = permute(data,[1 3 2]);
        data = abs(data);
        
        % create training, validation and test data sets:
        data_train = data(:,1:round(N/3),:);
        labels_train = labels(1:round(N/3));
        data_validation = data(:,(round(N/3)+1):2*round(N/3),:);
        labels_validation = labels((round(N/3)+1):2*round(N/3));
        data_test = data(:,(2*round(N/3)+1):end,:);
        labels_test = labels((2*round(N/3)+1):end);
        runICA = 0; % no ICA
        nClasses = 1; % binary classifier

    case 2
        if exist(filename) ~= 2
            error(['Cannot find the following EMG data: ' filename])
        end        
        disp('Loading EMG data...')
        windows_train = 500; % number of training samples
        windows_validation = 300; % number of test sample
        
        % load 6-category EMG data:
        data = load(filename);
        origdata = data.Data(1:48,:);
        labels = data.Labels;
        nClasses = 6;
        % position map:
        p1 = repmat((0:0.008:(0.008*11))',4,1);
        p2 = repmat((0:0.008:(0.008*3))',1,12)';p2 = p2(:);
        pos_map = [p1 p2];
        
        % **************************************************************
        % Here, we run Fourier-ICA:
        % **************************************************************
        runICA = 1; % estimate components
        pcadim = 30; % total number of components
        samplfreq = 2500; % sampling frequency in Hz
        minfreq = 20; % minimimum frequency in Hz
        maxfreq = 60; % maximum frequency in Hz
        windowlength_sec = 0.2; % window size in seconds
        overlapfactor = 1; % overlap between adjacent windows (1 means no overlap)
        
        % short-time Fourier transform:
        [X,Y,Lab] = extractWindows(origdata,labels,windowlength_sec,overlapfactor,samplfreq,minfreq,maxfreq);
        % run ICA:
        [data_all,A,W] = fouriericaCustom(X,pcadim,samplfreq,minfreq,maxfreq,'complexmixing',true,'maxiter',2000);
        data_all = abs(data_all); % take absolute value of the Fourier-coefficients
        
        % **************************************************************
        % Extract training, validation and test data sets:
        % **************************************************************
        data_train = data_all(:,1:windows_train,:);
        data_validation = data_all(:,(windows_train+1):(windows_train+windows_validation),:);
        data_test = data_all(:,(windows_train+windows_validation+1):end,:);
        
        labels_train = Lab(1:windows_train);
        labels_validation = Lab((windows_train+1):(windows_train+windows_validation));
        labels_test = Lab((windows_train+windows_validation+1):end);
    otherwise
        error('Unknown dataset!')
end

disp(['Training and validating classifier...'])
% **************************************************************
% Training and validation of sparse bilinear classifiers:
% **************************************************************

nrPairs = numel(lambdas_comp)*numel(lambdas_freq);
lambda_pairs = zeros(nrPairs,2);
lambdas_comp = sort(lambdas_comp,'descend');
validation_accuracy = zeros(1,nrPairs);
clear Classifier
iter = 1;
% train and validate classifier for each hyperparameter combination:
for m = 1:length(lambdas_comp)
    for n = 1:length(lambdas_freq)
        % train classifier:
        Classifier{iter} = train_sparse_bilinear(data_train,...
            labels_train,lambdas_freq(n),lambdas_comp(m),modelRank);
        % validate classifier:
        resu = test_sparse_bilinear(data_validation,labels_validation,Classifier{iter});
        % save validation accuracy and lambda values:
        validation_accuracy(iter) = resu.acc;
        lambda_pairs(iter,:) = [lambdas_freq(n) lambdas_comp(m)];
        iter = iter + 1;
    end
end

% Obtain the best model according to the validation results.
% To favor sparser solutions, we take sparsest classifier
% within 2 percent accuracy from maximum:
[bestValid,bestValidInd] = max(validation_accuracy);
bestValidInd = find(validation_accuracy>(bestValid - bestValid*0.02),1);
bestValid = validation_accuracy(bestValidInd);
bestLambdas = lambda_pairs(bestValidInd,:);
disp(['Selected hyperparameters: lambda1 = ' num2str(bestLambdas(1)) ', lambda2 = ' num2str(bestLambdas(2))])

% ****************************************************************
% Train final classifier:
% ****************************************************************
Classifier_final = train_sparse_bilinear(data_train,...
    labels_train,bestLambdas(1),bestLambdas(2),modelRank);
% evaluate final classification accuracy for independent test data set:
resu = test_sparse_bilinear(data_test,labels_test,Classifier_final);
disp(['Test data set classification accuracy: ' num2str(resu.acc*100) '%'])

% ****************************************************************
% Visualize spatial patterns of discriminative independent components:
% ****************************************************************
% Extract non-zero classification coefficients of the ICs:
if runICA
    [V,W] = organize_coefficients(Classifier_final.V,Classifier_final.W,modelRank);
    clear relevantCoefficients
    for k = 1:nClasses
        coefsW = W{k};
        for m = 1:modelRank 
            w = coefsW(m,:)';
            F = [find(w) w(find(w))]';
            relevantCoefficients{m}{k} = F;
        end
    end
    
    % Visualize spatial patterns corresponding to non-zero ICs. One set of 
    % components is plotted for each category (index k) and each coefficient 
    % vector (index m).
    for m = 1:modelRank
        for k = nClasses:-1:1
            if k == 1 && m == modelRank;plot_colorbar = 1;else;plot_colorbar = 0;end
            if modelRank > 1
                titl = ['Coefficient set ' num2str(m) ', discriminative components for category ' num2str(k)];
            else
                titl = ['Discriminative components for category ' num2str(k)];
            end            
            visualize_patterns(relevantCoefficients{m}{k},A,pos_map,plot_colorbar,titl);
        end
    end
end

Data.data_train = data_train;
Data.data_validation = data_validation;
Data.data_test = data_test;
Data.labels_train = labels_train;
Data.labels_validation = labels_validation;
Data.labels_test = labels_test;
