function [resu,Classifier_final] = train_and_test_spectral_LDA(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,params_spectral_LDA)
% Train, validate and test sparse Spectral LDA classifier.

% Inputs: 
% data_train - training data
% data_validation - validation data
% data_test - test data
% labels_train - class labels for training data
% labels_validation - class labels for validation data
% labels_test - class labels for test data
% params_spectral_LDA - struct containing classifier parameters:
%
% Data format: Data is a tensor of size C*N*F, where:
% C is the total number of channels or independent components 
% N is the total number of epochs 
% F is the number of frequency bins in each epoch.
%
% Class label format: labels are provided as a vector of length n 
% denoting class labels of each epoch. Categories are indexed as 
% 1,2,3,...,K, where K is the total number of categories.
% 
% Outputs:
%
% resu is a struct containing the test results:
% res.acc           Accuracy for the test data
% res.labelsEstim   Estimated class labels
% res.labelsTrue    True class labels
%
% Classifier_final is a struct containing classifier parameters 
% described in GLMNET:
% 
% Classifier_final.a0      Intercept sequence of length length(Classifier_final.lambda).
% Classifier_final.beta    For "elnet" and "lognet" models, a nvars x length(lambda) matrix of coefficients. For "multnet", a list of nc such matrices, one for each class.
% Classifier_final.lambda  The actual sequence of lambda values used.
% Classifier_final.dev     The fraction of (null) deviance explained (for "elnet", this is the R-square).
% Classifier_final.nulldev Null deviance (per observation).
% Classifier_final.df      The number of nonzero coefficients for each value of lambda. For "multnet", this is the number of variables with a nonzero coefficient for any class.
% Classifier_final.dfmat   For "multnet" only. A matrix consisting of the number of nonzero coefficients per class.
% Classifier_final.dim     Dimension of coefficient matrix (ices).
% Classifier_final.npasses Total passes over the data summed over all lambda values.
% Classifier_final.offset  a logical variable indicating whether an offset was included in the model.
% Classifier_final.jerr    Error flag, for warnings and errors (largely for internal debugging).
% Classifier_final.class   Type of regression - internal usage.
% Classifier_final.call    a cell including the names of all the input variables in the parent environment.

% Extract spectral LDA features:
[U,V] = estimate_projections(data_train,labels_train,'spectral_LDA');
features_train = project_data(data_train,[],V,'spectral_LDA');
features_validation = project_data(data_validation,[],V,'spectral_LDA');
if isstruct(data_test)
    features_test.data1 = project_data(data_test.data1,[],V,'spectral_LDA');
    features_test.data2 = project_data(data_test.data2,[],V,'spectral_LDA');    
else
    features_test = project_data(data_test,[],V,'spectral_LDA');    
end
% Train and validate classifiers for several hyperparameters:
options = glmnetSet;
options.standardize = 1;
close all
[labels_train,features_train] = balance_labels(labels_train,features_train);
[labels_validation,features_validation] = balance_labels(labels_validation,features_validation);

Classifier1 = train_sparse_linear(features_train,labels_train,options);
options.lambda = Classifier1.lambda;
options.nlambda = [];
rresu1 = test_sparse_linear(Classifier1,features_validation,labels_validation);

Classifier2 = train_sparse_linear(features_validation,labels_validation,options);
rresu2 = test_sparse_linear(Classifier2,features_train,labels_train);

validation_accuracy = [rresu1.acc;rresu2.acc];
val_acc_mean = mean(validation_accuracy,1);
val_acc_std = std(validation_accuracy,[],1);
SE = val_acc_std/sqrt(2);
% Obtain the best regularization hyperparameter according to the validation results:
[ma,mai] = max(val_acc_mean);
ma = max(val_acc_mean) - 3*SE;
fi = find(val_acc_mean > ma);
if isempty(fi)
   fi = 1; 
end
bestValidInd = fi(1);
%figure,plot(val_acc_mean)
%[bestValid,bestValidInd]

%options.lambda = Classifier.lambda(bestValidInd);

% Train the final classifier using both training+validation data:
options.standardize = 1;
Classifier_final = train_sparse_linear([features_train;features_validation],[labels_train labels_validation],options);
if bestValidInd > length(Classifier_final.lambda)
    bestValidInd = length(Classifier_final.lambda);
end
Classifier_final.bestClassifierIndex = bestValidInd;
Classifier_final.spectralProjections = V;

% Test final classifier:
if isstruct(features_test)
    data_test1 = features_test.data1;
    data_test2 = features_test.data2;
    labels_test1 = labels_test.labels1;
    labels_test2 = labels_test.labels2;
    
    [labels_test1,data_test1,data_test.data1,test_data_inds1] = balance_labels(labels_test1,data_test1,data_test.data1);
    [labels_test2,data_test2,data_test.data2,test_data_inds2] = balance_labels(labels_test2,data_test2,data_test.data2);
    
    res1 = test_sparse_linear(Classifier_final,data_test1,labels_test1);
    res2 = test_sparse_linear(Classifier_final,data_test2,labels_test2);
    resu.acc1 = res1.acc(bestValidInd);
    resu.proj1 = res1.proj(:,bestValidInd);
    resu.labelsTrue1 = res1.labelsTrue;   
    resu.labelsEstim1 = res1.labelsEstim(:,bestValidInd);
    resu.acc_seq1 = res1.acc;
    resu.proj2 = res2.proj(:,bestValidInd);
    resu.acc2 = res2.acc(bestValidInd);
    resu.labelsTrue2 = res2.labelsTrue;   
    resu.labelsEstim2 = res2.labelsEstim(:,bestValidInd);
    resu.acc_seq2 = res2.acc;
    resu.res1 = res1;
    resu.res2 = res2;
    resu.test_data_inds1 = test_data_inds1;
    resu.test_data_inds2 = test_data_inds2;    
    resu.features_test = features_test;
    resu.data_test = data_test;
    resu.features_test1 = data_test1;
    resu.features_test2 = data_test2;
    resu.labels_test1 = labels_test1;
    resu.labels_test2 = labels_test2;
else
    [labels_test,features_test] = balance_labels(labels_test,features_test);
    res = test_sparse_linear(Classifier_final,features_test,labels_test);
    resu.acc = res.acc(bestValidInd);
    resu.proj = res.proj(:,bestValidInd);
    resu.labelsTrue = res.labelsTrue;   
    resu.labelsEstim = res.labelsEstim(:,bestValidInd);
    resu.acc_seq = res.acc;
    resu.res = res;
    resu.features_test = features_test;
    resu.data_test = data_test;
end



